<?php

/**
 * @file
 * Definition of Drupal\views_accordion\Plugin\views\style\ViewsAccordion.
 */

namespace Drupal\views_accordion\Plugin\views\style;

use Drupal\Core\Form\FormStateInterface;
use Drupal\views\Plugin\views\style\StylePluginBase;

/**
 * Style plugin to render each item in an ordered or unordered list.
 *
 * @ingroup views_style_plugins
 *
 * @ViewsStyle(
 *   id = "views_accordion",
 *   title = @Translation("jQuery UI accordion"),
 *   help = @Translation("Display a JQuery accordion with the results. The first field will be used as the header and trigger."),
 *   theme = "views_view_unformatted",
 *   display_types = {"normal"}
 * )
 */
class ViewsAccordion extends StylePluginBase {
  /**
   * Overrides \Drupal\views\Plugin\views\style\StylePluginBase::usesRowPlugin
   */
  protected $usesRowPlugin = TRUE;

  /**
  * Overrides \Drupal\views\Plugin\views\style\StylePluginBase::usesRowClass.
  */
  protected $usesRowClass = TRUE;

  /**
  * Overrides \Drupal\views\Plugin\views\style\StylePluginBase::groupingTheme.
  */
  // protected $groupingTheme = 'views_view_grouping';

  /**
   * Set default options
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    //dpm($options);

    $options['use-grouping-header'] = ['default' => 0];
    $options['collapsible'] = ['default' => 0];
    $options['row-start-open'] = ['default' => 0];
    $options['animated'] = ['default' => 'slide'];
    $options['autoheight'] = ['default' => 1];
    $options['event'] = ['default' => 'click'];
    $options['fillspace'] = ['default' => 0];
    $options['navigation'] = ['default' => 0];
    $options['clearstyle'] = ['default' => 0];
    $options['disableifone'] = ['default' => 0];

    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);

    //// available valid options for grouping (used for use-grouping-header #dependency)
    foreach ($this->displayHandler->getHandlers('field') as $field => $handler) {
      $options[] = $field;
    }
    // Find out how many items the display is currently configured to show (row-start-open)
    $maxitems = $this->displayHandler->getOption('items_per_page');
    $maxitems = ($maxitems == 0) ? 10 : $maxitems;  // if items_per_page is set to unlimitted (0), 10 rows will be what the user gets to choose from.

    // Setup our array of options for choosing which row should start opened (row-start-open)
    $rsopen_options = [];
    for ($i = 1; $i <= $maxitems; $i++){
      $rsopen_options[] = t('Row ') . $i;
    }
    $rsopen_options['none'] = t('None');

    /*
     * See /core/core.libraries.yml and http://api.jqueryui.com/1.10/easings/
     * We've used all the easing effects form ui effects core.
     */
    $animated_options = [
      'none' => t('None'),
      'linear' => t('Linear'),
      'swing' => t('Swing'),
      'easeInQuart' => t('easeInQuart'),
      'easeOutQuart' => t('easeOutQuart'),
      'easeInOutQuart' => t('easeInOutQuart'),
      'easeInExpo' => t('easeInExpo'),
      'easeOutExpo' => t('easeOutExpo'),
      'easeInOutExpo' => t('easeInOutExpo'),
      'easeInBack' => t('easeInBack'),
      'easeOutBack' => t('easeOutBack'),
      'easeInOutBack' => t('easeInOutBack'),
      'easeInQuad' => t('easeInQuad'),
      'easeOutQuad' => t('easeOutQuad'),
      'easeInOutQuad' => t('easeInOutQuad'),
      'easeInQuint' => t('easeInQuint'),
      'easeOutQuint' => t('easeOutQuint'),
      'easeInOutQuint' => t('easeInOutQuint'),
      'easeInCirc' => t('easeInCirc'),
      'easeOutCirc' => t('easeOutCirc'),
      'easeInOutCirc' => t('easeInOutCirc'),
      'easeInBounce' => t('easeInBounce'),
      'easeOutBounce' => t('easeOutBounce'),
      'easeInOutBounce' => t('easeInOutBounce'),
      'easeInCubic' => t('easeInCubic'),
      'easeOutCubic' => t('easeOutCubic'),
      'easeInOutCubic' => t('easeInOutCubic'),
      'easeInSine' => t('easeInSine'),
      'easeOutSine' => t('easeOutSine'),
      'easeInOutSine' => t('easeInOutSine'),
      'easeInElastic' => t('easeInElastic'),
      'easeOutElastic' => t('easeOutElastic'),
      'easeInOutElastic' => t('easeInOutElastic'),
    ];

    $c = count($this->options['grouping']);
    // Add a form use group header field for every grouping, plus one.
    for ($i = 0; $i <= $c; $i++) {
      $grouping = !empty($this->options['grouping'][$i]) ? $this->options['grouping'][$i] : [];
      $grouping += ['use-grouping-header' => 0];
      $form['grouping'][$i]['use-grouping-header'] = [
        '#type' => 'checkbox',
        '#title' => t('Use the group header as the Accordion header'),
        '#default_value' => $grouping['use-grouping-header'],
        '#description' => t('If checked, the Group\'s header will be used to open/close the accordion.'),
        '#states' => [
          'invisible' => [
            ':input[name="style_options[grouping][' . $i . '][field]"]' => ['value' => ''],
          ],
        ],
      ];
    }
    $form['grouping']['#prefix'] = '<div class="form-item">'. t('<strong>IMPORTANT:</strong> The <em>first field</em> in order of appearance <em>will</em> be the one used as the "header" or "trigger" of the accordion action.') .'</div>';

    $form['row-start-open'] = [
      '#type' => 'select',
      '#title' => t('Row to display opened on start'),
      '#default_value' => $this->options['row-start-open'],
      '#description' => t('Choose which row should start opened when the accordion first loads. If you want all to start closed, choose "None", and make sure to have "Allow for all rows to be closed" on below.'),
      '#options' => $rsopen_options,
    ];
    $form['collapsible'] = [
      '#type' => 'checkbox',
      '#title' => t('Allow for all rows to be closed'),
      '#default_value' => $this->options['collapsible'],
      '#description' => t('If you check this on, when the user clicks on an opened item, it will close.'),
    ];
    $form['animated'] = [
      '#type' => 'select',
      '#title' => t('Animation effect'),
      '#default_value' => $this->options['animated'],
      '#description' => t('Choose what animation effect you would like to see, or "None" to disable it.'),
      '#options' => $animated_options,
    ];
    $form['animation_time'] = [
      '#type' => 'number',
      '#title' => t('Animation time'),
      '#default_value' => empty($this->options['animation_time']) ? 300 : $this->options['animation_time'],
      '#min' => 0,
      '#step' => 1,
      '#description' => t('The animation duration in milliseconds'),
    ];
     $form['autoheight'] = [
      '#type' => 'checkbox',
      '#title' => t('Autoheight'),
      '#default_value' => $this->options['autoheight'],
      '#description' => t('If set, the highest content part is used as height reference for all other parts. Provides more consistent animations.'),
    ];
    $form['event'] = [
      '#type' => 'select',
      '#title' => t('Event'),
      '#default_value' => $this->options['event'],
      '#description' => t('The event on which to trigger the accordion.'),
      '#options' => [
        'click' => t('Click'),
        'mouseover' => t('Mouseover'),
      ],
    ];
    $form['fillspace'] = [
      '#type' => 'checkbox',
      '#title' => t('Fillspace'),
      '#default_value' => $this->options['fillspace'],
      '#description' => t('If set, the accordion completely fills the height of the parent element. Overrides autoheight.'),
    ];
    $form['navigation'] = [
      '#type' => 'checkbox',
      '#title' => t('Navigation'),
      '#default_value' => $this->options['navigation'],
      '#description' => t('If set, looks for the anchor that matches location.href and activates it. Great for href-based state-saving. Use navigationFilter to implement your own matcher.'),
    ];
    $form['clearstyle'] = [
      '#type' => 'checkbox',
      '#title' => t('Clearstyle'),
      '#default_value' => $this->options['clearstyle'],
      '#description' => t("If set, clears height and overflow styles after finishing animations. This enables accordions to work with dynamic content. Won't work together with autoHeight."),
    ];
    $form['disableifone'] = [
      '#type' => 'checkbox',
      '#title' => t('Disable if only one result'),
      '#default_value' => $this->options['disableifone'],
      '#description' => t("If set, the accordion will not be shown when there are less than 2 results."),
    ];
  }

  public function preRender($result) {
    // no need do anything if we are using the grouped field as the header
    if ($this->options['use-grouping-header']){
      return;
    }
    // no need do anything if we we have only one result and disableifone is active
    if ($this->options['disableifone'] == '1') {
      if(count($result) < 2) {
        return;
      }
    }

    // Find out about the header field options
    $fields = array_values($this->displayHandler->getOption('fields'));
    $header_class = 'views-accordion-header';
    // Add header class to first not-excluded field
    foreach ($fields as $field) {
      if (!isset($field['exclude']) || ($field['exclude'] == 0)) {
        // setup our wrapper class
        // if the user configured its own class, set that up with our own class
        if(!empty($field['element_wrapper_class'])) {
          $header_wrapper_class = $field['element_wrapper_class'] .' '. $header_class;
        }
        // otherwise use our own class only
        else {
          $header_wrapper_class = $header_class;
        }
        // setup the view to use our processed wrapper class
        $this->view->field[$field['id']]->options['element_wrapper_class'] = $header_wrapper_class;
        // make sure we are using a div for markup at least
        if (empty($field['element_wrapper_type'])) {
            $this->view->field[$field['id']]->options['element_wrapper_type'] = 'div';
        }
        break;
      }
    }
    $this->view->element['#attached']['library'][] = 'views_accordion/views_accordion.accordion';

    // Add the appropiate effect library if necessary
    $effect = $this->options['animated'];
    if(($effect !== 'none') && ($effect !== 'swing') && ($effect !== 'linear')){
      // For now we only use ui core effects library, which provides the easing effects.
      // this switch is left here in case we want to integrate and load any other libraries
      switch($effect) {
          default:
            $library = 'core/jquery.ui.effects.core';
            break;
      }
      if(isset($library)){
        $this->view->element['#attached']['library'][] = $library;
      }
    }

    // Preparing the js variables and adding the js to our display
    // we do it here so we dont have it run once every group
    $view_settings['collapsible'] = $this->options['collapsible'];
    $view_settings['rowstartopen'] = ($this->options['row-start-open'] == 'none') ? FALSE : $this->options['row-start-open'] + 0; // forces it to add it as an integrer instead of a string - ugly hack?
    $view_settings['animated'] = ($this->options['animated'] == 'none') ? FALSE : $this->options['animated'];
    $view_settings['duration'] = ($this->options['animated'] == 'none') ? FALSE : $this->options['animation_time'];
    $view_settings['autoheight'] = $this->options['autoheight'];
    $view_settings['event'] = $this->options['event'];
    $view_settings['fillspace'] = $this->options['fillspace'];
    $view_settings['navigation'] = $this->options['navigation'];
    $view_settings['clearstyle'] = $this->options['clearstyle'];

    $view_settings['grouping'] =  $this->options['grouping'] ? 1 : 0;

    $view_settings['display'] =  $this->view->current_display;
    $view_settings['viewname'] = $this->view->storage->id();
    $view_settings['usegroupheader'] = $view_settings['grouping'] ? $this->options['use-grouping-header'] : 0;

    $accordion_id = 'views-accordion-'. $view_settings['viewname'] .'-'. $this->view->current_display;

    if ($view_settings['usegroupheader'] == 1) {
      // @todo we cannot set this class in the backend apparently...
      $view_settings['header'] = 'h3.'. $accordion_id .'-header';
    }

    // Used to get the first field to be used as the accordion header
    if ($view_settings['usegroupheader'] == 0) {
      $view_settings['header'] = '.views-accordion-header';
    }
    $this->view->element['#attached']['drupalSettings']['views_accordion'] = [$accordion_id => $view_settings];
  }

  /**
   * Overrides \Drupal\views\Plugin\views\style\StylePluginBase\StylePluginBase::render().
   */
  public function render() {
    $output = '';

    // Prevent errors if users select content instead of fields as row style
    // This forces them to use field as row style, no other way that I can think of
    if(parent::usesFields()){
      $rows = parent::render();
      $output = [
        '#theme' => $this->themeFunctions(),
        '#view' => $this->view,
        '#options' => $this->options,
        '#rows' => $rows
      ];
    }
    else {
      drupal_set_message('Views accordion requires Fields as row style', 'error');
    }

    return $output;
  }
}
